from Packages import *

class LinearPCA:
    def __init__(self, X, Ncomp=10):
        ## input matrix
        self.__x = np.array(X)
        ## set the number of components to compute and fit the model
        self.__ncp = Ncomp

        # Fit PCA model
        M = PCA(n_components = self.__ncp)
        M.fit(self.__x)

        ######## results ########        
        # Results
        self.__pcnames = [f'PC{i+1}({100 *  M.explained_variance_ratio_[i].round(2)}%)' for i in range(self.__ncp)]
        self._Qexp_ratio = pd.DataFrame(100 *  M.explained_variance_ratio_, columns = ["Qexp"], index= [f'PC{i+1}' for i in range(self.__ncp)])

        self._p = M.components_.T
        self._t = M.transform(self.__x)
        self.eigvals = M.singular_values_**2
        self.Lambda = np.diag(self.eigvals)

        # Matrix reconstruction or prediction making
        self.T2 = {}
        self._xp = {}
        self._qres = {}
        self.leverage = {}
        
        # 
        for i in range(self.__ncp):
            # Matrix reconstruction- prediction
            self._xp[i] = np.dot(self._t[:,:i+1], self._p.T[:i+1,:])

            # Q residuals: Q residuals represent the magnitude of the variation remaining in each sample after projection through the model
            self._qres[i] = np.diag(np.subtract(self.__x, self._xp[i])@ np.subtract(self.__x, self._xp[i]).T)
            self.T2[i] = np.diag(self._t[:,:i+1] @ np.transpose(self._t[:,:i+1]))

            # Laverage
            Hat =  self._t[:,:i+1] @ np.linalg.inv(np.transpose(self._t[:,:i+1]) @ self._t[:,:i+1]) @ np.transpose(self._t[:,:i+1])
            self.leverage[i] = np.diag(Hat) / np.trace(Hat)


    @property
    def scores_(self):
        return pd.DataFrame(self._t, columns= self.__pcnames)
    
    @property
    def loadings_(self):
        return pd.DataFrame(self._p, columns=self.__pcnames)
    
    @property
    def leverage_(self):
        lev = pd.DataFrame(self.leverage)
        lev.columns =self.__pcnames
        return lev
    
    @property
    def residuals_(self):
        res = pd.DataFrame(self._qres)
        res.columns=self.__pcnames
        return res
    
    @property
    def hotelling_(self):
        hot = pd.DataFrame(self.T2)
        hot.columns=self.__pcnames
        return hot